@@ -29,6 +29,8 @@
 #include "pango-fontmap.h"
 #include "pango-impl-utils.h"
 
+static const char bad_font_warning[] = "%s called with bad font, expect ugly output";
+
 struct _PangoFontDescription
 {
   char *family_name;
@@ -346,6 +348,7 @@ pango_font_description_set_size (PangoFontDescription *desc,
 				 gint                  size)
 {
   g_return_if_fail (desc != NULL);
+  g_return_if_fail (size >= 0);
   
   desc->size = size;
   desc->size_is_absolute = FALSE;
@@ -362,7 +365,8 @@ pango_font_description_set_size (PangoFontDescription *desc,
  * Return value: the size field for the font description in points or device units.
  *   You must call pango_font_description_get_size_is_absolute()
  *   to find out which is the case. Returns 0 if the size field has not
- *   previously been set.  pango_font_description_get_set_fields() to
+ *   previously been set or it has been set to 0 explicitly.
+ *   Use pango_font_description_get_set_fields() to
  *   find out if the field was explicitely set or not.
  **/
 gint
@@ -376,7 +380,7 @@ pango_font_description_get_size (const PangoFontDescription *desc)
 /**
  * pango_font_description_set_absolute_size:
  * @desc: a #PangoFontDescription
- * @size: the new size, in Pango units. There are #PANGO_SCALE Pango units in one
+ * @size: the new size, in Pango units. There are %PANGO_SCALE Pango units in one
  *   device unit. For an output backend where a device unit is a pixel, a @size
  *   value of 10 * PANGO_SCALE gives a 10 pixel font.
  * 
@@ -390,6 +394,7 @@ pango_font_description_set_absolute_size (PangoFontDescription *desc,
 					  double                size)
 {
   g_return_if_fail (desc != NULL);
+  g_return_if_fail (size >= 0);
   
   desc->size = size;
   desc->size_is_absolute = TRUE;
@@ -797,7 +802,8 @@ find_field (const FieldMap *map, int n_elements, const char *str, int len, int *
   
   for (i=0; i<n_elements; i++)
     {
-      if (map[i].str && g_ascii_strncasecmp (map[i].str, str, len) == 0)
+      if (map[i].str[0] && g_ascii_strncasecmp (map[i].str, str, len) == 0 &&
+	  map[i].str[len] == '\0')
 	{
 	  if (val)
 	    *val = map[i].value;
@@ -842,16 +848,23 @@ getword (const char *str, const char *last, size_t *wordlen)
 static gboolean
 parse_size (const char *word,
 	    size_t      wordlen,
-	    int        *pango_size)
+	    int        *pango_size,
+	    gboolean   *size_is_absolute)
 {
   char *end;
   double size = g_ascii_strtod (word, &end);
 
-  if ((size_t)(end - word) == wordlen) /* word is a valid float */
+  if (end != word &&
+      (end == word + wordlen ||
+       (end + 2 == word + wordlen && !strncmp (end, "px", 2))
+      ) && size >= 0 && size <= 1000000) /* word is a valid float */
     {
       if (pango_size)
 	*pango_size = (int)(size * PANGO_SCALE + 0.5);
 
+      if (size_is_absolute)
+        *size_is_absolute = end < word + wordlen;
+
       return TRUE;
     }
 
@@ -867,7 +880,8 @@ parse_size (const char *word,
  * comma separated list of families optionally terminated by a comma,
  * STYLE_OPTIONS is a whitespace separated list of words where each
  * WORD describes one of style, variant, weight, or stretch, and SIZE
- * is an decimal number (size in points). Any one of the options may
+ * is a decimal number (size in points) or optionally followed by the
+ * unit modifier "px" for absolute size. Any one of the options may
  * be absent.  If FAMILY-LIST is absent, then the family_name field of
  * the resulting font description will be initialized to %NULL.  If
  * STYLE-OPTIONS is missing, then all style options will be set to the
@@ -911,8 +925,10 @@ pango_font_description_from_string (const char *str)
    */
   if (wordlen != 0)
     {
-      if (parse_size (p, wordlen, &desc->size))
+      gboolean size_is_absolute;
+      if (parse_size (p, wordlen, &desc->size, &size_is_absolute))
 	{
+	  desc->size_is_absolute = size_is_absolute;
 	  desc->mask |= PANGO_FONT_MASK_SIZE;
 	  last = p;
 	}
@@ -1010,7 +1026,7 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
       p = getword (desc->family_name, desc->family_name + strlen(desc->family_name), &wordlen);
       if (wordlen != 0 &&
 	  (find_field_any (p, wordlen, NULL) ||
-	   (parse_size (p, wordlen, NULL) &&
+	   (parse_size (p, wordlen, NULL, NULL) &&
 	    desc->weight == PANGO_WEIGHT_NORMAL &&
 	    desc->style == PANGO_STYLE_NORMAL &&
 	    desc->stretch == PANGO_STRETCH_NORMAL &&
@@ -1036,6 +1052,9 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
 
       g_ascii_dtostr (buf, sizeof (buf), (double)desc->size / PANGO_SCALE);
       g_string_append (result, buf);
+
+      if (desc->size_is_absolute)
+	g_string_append (result, "px");
     }
   
   return g_string_free (result, FALSE);
@@ -1087,7 +1106,9 @@ pango_font_init (PangoFont *font)
  * pango_font_describe:
  * @font: a #PangoFont
  * 
- * Returns a description of the font.
+ * Returns a description of the font, with font size set in points.
+ * Use pango_font_describe_with_absolute_size() if you want the font
+ * size in device units.
  * 
  * Return value: a newly-allocated #PangoFontDescription object.
  **/
@@ -1099,6 +1120,30 @@ pango_font_describe (PangoFont      *font)
   return PANGO_FONT_GET_CLASS (font)->describe (font);
 }
 
+/**
+ * pango_font_describe_with_absolute_size:
+ * @font: a #PangoFont
+ * 
+ * Returns a description of the font, with absolute font size set
+ * (in device units). Use pango_font_describe() if you want the font
+ * size in points.
+ * 
+ * Return value: a newly-allocated #PangoFontDescription object.
+ **/
+PangoFontDescription *
+pango_font_describe_with_absolute_size (PangoFont      *font)
+{
+  g_return_val_if_fail (font != NULL, NULL);
+
+  if (G_UNLIKELY (!PANGO_FONT_GET_CLASS (font)->describe_absolute))
+    {
+      g_warning ("describe_absolute not implemented for this font class, report this as a bug");
+      return pango_font_describe (font);
+    }
+
+  return PANGO_FONT_GET_CLASS (font)->describe_absolute (font);
+}
+
 /**
  * pango_font_get_coverage:
  * @font: a #PangoFont
@@ -1171,7 +1216,7 @@ pango_font_get_glyph_extents  (PangoFont      *font,
       if (!_pango_warning_history.get_glyph_extents)
         {
 	  _pango_warning_history.get_glyph_extents = TRUE;
-	  g_warning ("pango_font_get_glyph_extents called with bad font, expect ugly output");
+	  g_warning (bad_font_warning, "pango_font_get_glyph_extents");
 	}
       if (ink_rect)
         {
@@ -1212,6 +1257,17 @@ PangoFontMetrics *
 pango_font_get_metrics (PangoFont        *font,
 			PangoLanguage    *language)
 {
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_metrics)
+        {
+	  _pango_warning_history.get_metrics = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_metrics");
+	}
+      return pango_font_metrics_new ();
+    }
+
   return PANGO_FONT_GET_CLASS (font)->get_metrics (font, language);
 }
 
@@ -1228,6 +1284,17 @@ pango_font_get_metrics (PangoFont        *font,
 PangoFontMap *
 pango_font_get_font_map (PangoFont *font)
 {
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_font_map)
+        {
+	  _pango_warning_history.get_font_map = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_font_map");
+	}
+      return NULL;
+    }
+
   if (PANGO_FONT_GET_CLASS (font)->get_font_map)
     return PANGO_FONT_GET_CLASS (font)->get_font_map (font);
   else
@@ -1317,7 +1384,7 @@ pango_font_metrics_unref (PangoFontMetrics *metrics)
  * actual drawn ink. It is necessary to lay out the text to figure
  * where the ink will be.)
  * 
- * Return value: the ascent, in Pango units. (1 point == #PANGO_SCALE Pango units.)
+ * Return value: the ascent, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_ascent (PangoFontMetrics *metrics)
@@ -1337,7 +1404,7 @@ pango_font_metrics_get_ascent (PangoFontMetrics *metrics)
  * actual drawn ink. It is necessary to lay out the text to figure
  * where the ink will be.)
  * 
- * Return value: the descent, in Pango units. (1 point == #PANGO_SCALE Pango units.)
+ * Return value: the descent, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_descent (PangoFontMetrics *metrics)
@@ -1356,7 +1423,7 @@ pango_font_metrics_get_descent (PangoFontMetrics *metrics)
  * determining the initial size for a window. Actual characters in
  * text will be wider and narrower than this.
  * 
- * Return value: the character width, in Pango units. (1 point == #PANGO_SCALE Pango units.)
+ * Return value: the character width, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_approximate_char_width (PangoFontMetrics *metrics)
@@ -1373,11 +1440,11 @@ pango_font_metrics_get_approximate_char_width (PangoFontMetrics *metrics)
  * Gets the approximate digit width for a font metrics structure.
  * This is merely a representative value useful, for example, for
  * determining the initial size for a window. Actual digits in
- * text can be wider and narrower than this, though this value
+ * text can be wider or narrower than this, though this value
  * is generally somewhat more accurate than the result of
- * pango_font_metrics_get_approximate_digit_width().
+ * pango_font_metrics_get_approximate_char_width() for digits.
  * 
- * Return value: the digit width, in Pango units. (1 point == #PANGO_SCALE Pango units.)
+ * Return value: the digit width, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_approximate_digit_width (PangoFontMetrics *metrics)
@@ -1639,7 +1706,8 @@ pango_font_face_list_sizes (PangoFontFace  *face,
     PANGO_FONT_FACE_GET_CLASS (face)->list_sizes (face, sizes, n_sizes);
   else
     {
-      *sizes = NULL;
+      if (sizes != NULL)
+	*sizes = NULL;
       *n_sizes = 0;
     }
 }
