@@ -1157,7 +1157,7 @@ static const char canon_map[256] = {
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,  '-',  0,   0, 
-   0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
+  '0', '1', '2', '3', '4', '5', '6', '7',  '8', '9',  0,   0,   0,   0,   0,   0, 
    0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
   'p', 'q', 'r', 's', 't', 'u', 'v', 'w',  'x', 'y', 'z',  0,   0,   0,   0,  '-',
    0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
@@ -1171,17 +1171,12 @@ lang_equal (gconstpointer v1,
   const guchar *p1 = v1;
   const guchar *p2 = v2;
 
-  while (*p1 && *p2)
+  while (canon_map[*p1] && canon_map[*p1] == canon_map[*p2])
     {
-      guchar c1 = canon_map[*p1];
-      guchar c2 = canon_map[*p2];
-      if (!c1) { p1++; continue; }
-      if (!c2) { p2++; continue; }
-      if (c1 != c2) return FALSE;
       p1++, p2++;
     }
 
-  return (*p1 == *p2);
+  return (canon_map[*p1] == canon_map[*p2]);
 }
 
 static guint
@@ -1189,11 +1184,9 @@ lang_hash (gconstpointer key)
 {
   const guchar *p = key;
   guint h = 0;
-  while (*p)
+  while (canon_map[*p])
     {
-      guchar value = canon_map[*p];
-      if (value)
-	h = (h << 5) - h + value;
+      h = (h << 5) - h + canon_map[*p];
       p++;
     }
 
@@ -1261,13 +1254,8 @@ pango_language_from_string (const char *language)
   result = g_malloc (len + 1);
 
   p = result;
-  while (*language)
-    {
-      char value = canon_map[*(guchar *)language++];
-      if (value)
-	*(p++) = value;
-    }
-  *p++ = '\0';
+  while ((*(p++) = canon_map[*(guchar *)language++]))
+    ;
 
   g_hash_table_insert (hash, result, result);
 
@@ -1278,15 +1266,16 @@ pango_language_from_string (const char *language)
  * pango_language_matches:
  * @language: a language tag (see pango_language_from_string()),
  *            %NULL is allowed and matches nothing but '*'
- * @range_list: a list of language ranges, separated by ';' characters.
- *   each element must either be '*', or a RFC 3066 language range
+ * @range_list: a list of language ranges, separated by ';', ':',
+ *   ',', or space characters.
+ *   Each element must either be '*', or a RFC 3066 language range
  *   canonicalized as by pango_language_from_string()
  * 
  * Checks if a language tag matches one of the elements in a list of
  * language ranges. A language tag is considered to match a range
  * in the list if the range is '*', the range is exactly the tag,
- * or the range is a prefix of the tag, and the character after the
- * tag is '-'.
+ * or the range is a prefix of the tag, and the character after it
+ * in the tag is '-'.
  *
  * Return value: %TRUE if a match was found.
  **/
@@ -1300,7 +1289,7 @@ pango_language_matches (PangoLanguage *language,
 
   while (!done)
     {
-      const char *end = strchr (p, ';');
+      const char *end = strpbrk (p, ";:, \t");
       if (!end)
 	{
 	  end = p + strlen (p);
@@ -1766,7 +1755,7 @@ pango_find_base_dir (const gchar *text,
   PangoDirection dir = PANGO_DIRECTION_NEUTRAL;
   const gchar *p;
 
-  g_return_val_if_fail (text != NULL, PANGO_DIRECTION_NEUTRAL);
+  g_return_val_if_fail (text != NULL || length == 0, PANGO_DIRECTION_NEUTRAL);
 
   p = text;
   while ((length < 0 || p < text + length) && *p)
@@ -1799,25 +1788,39 @@ gboolean
 pango_is_zero_width (gunichar ch)
 {
 /* Zero Width characters:
+ *
+ *  00AD  SOFT HYPHEN
+ *  034F  COMBINING GRAPHEME JOINER
  *
  *  200B  ZERO WIDTH SPACE
  *  200C  ZERO WIDTH NON-JOINER
  *  200D  ZERO WIDTH JOINER
  *  200E  LEFT-TO-RIGHT MARK
  *  200F  RIGHT-TO-LEFT MARK
+ *
  *  2028  LINE SEPARATOR
+ *
  *  202A  LEFT-TO-RIGHT EMBEDDING
  *  202B  RIGHT-TO-LEFT EMBEDDING
  *  202C  POP DIRECTIONAL FORMATTING
  *  202D  LEFT-TO-RIGHT OVERRIDE
  *  202E  RIGHT-TO-LEFT OVERRIDE
+ *
+ *  2060  WORD JOINER
+ *  2061  FUNCTION APPLICATION
+ *  2062  INVISIBLE TIMES
+ *  2063  INVISIBLE SEPARATOR
+ *
  *  FEFF  ZERO WIDTH NO-BREAK SPACE
  */
-  return ((ch & ~(gunichar)0x003F) == 0x2000 && (
+  return ((ch & ~(gunichar)0x007F) == 0x2000 && (
 		(ch >= 0x200B && ch <= 0x200F) ||
-		ch == 0x2028 ||
-		(ch >= 0x202A && ch <= 0x202E)
-	 )) || ch == 0xFEFF;
+		(ch >= 0x202A && ch <= 0x202E) ||
+		(ch >= 0x2060 && ch <= 0x2063) ||
+		(ch == 0x2028)
+	 )) || G_UNLIKELY (ch == 0x00AD
+			|| ch == 0x034F
+			|| ch == 0xFEFF);
 }
 
 /**
