@@ -24,7 +24,7 @@
 
 #include "pango-fontmap.h"
 #include "pangoatsui-private.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 #include "modules.h"
 
 #import <Cocoa/Cocoa.h>
@@ -545,7 +545,7 @@ pango_atsui_font_map_load_font (PangoFontMap               *fontmap,
 
   size = pango_font_description_get_size (description);
 
-  if (size <= 0)
+  if (size < 0)
     return NULL;
 
   name = g_utf8_casefold (pango_font_description_get_family (description), -1);
@@ -578,9 +578,6 @@ pango_atsui_font_map_load_font (PangoFontMap               *fontmap,
 	    }
 	}
       
-      if (best_desc == NULL || best_face == NULL)
-        return NULL;
-
       pango_font_description_set_size (best_desc, size);
 
       best_font = pango_atsui_font_map_lookup (atsuifontmap, context, best_desc, best_face->postscript_name);
