@@ -229,12 +229,12 @@ get_context_info (PangoContext *context,
   static GQuark context_info_quark;
   PangoCairoContextInfo *info;
 
-  if (!context_info_quark)
+  if (G_UNLIKELY (!context_info_quark))
     context_info_quark = g_quark_from_static_string ("pango-cairo-context-info");
 
   info = g_object_get_qdata (G_OBJECT (context), context_info_quark);
 
-  if (!info && create)
+  if (G_UNLIKELY (!info) && create)
     {
       info = g_slice_new (PangoCairoContextInfo);
       info->dpi = -1.0;
@@ -343,13 +343,6 @@ pango_cairo_context_get_resolution (PangoContext *context)
     return -1.0;
 }
 
-struct _cairo_font_options {
-    cairo_antialias_t antialias;
-    cairo_subpixel_order_t subpixel_order;
-    cairo_hint_style_t hint_style;
-    cairo_hint_metrics_t hint_metrics;
-};
-
 /**
  * pango_cairo_context_set_font_options:
  * @context: a #PangoContext, from pango_cairo_font_map_create_context()
