@@ -173,12 +173,9 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
   cairo_glyph_t *cairo_glyphs;
   cairo_glyph_t stack_glyphs[MAX_STACK];
 
+  cairo_save (crenderer->cr);
   if (!crenderer->do_path)
-    {
-      cairo_save (crenderer->cr);
-
-      set_color (crenderer, PANGO_RENDER_PART_FOREGROUND);
-    }
+    set_color (crenderer, PANGO_RENDER_PART_FOREGROUND);
 
   if (!_pango_cairo_font_install (PANGO_CAIRO_FONT (font), crenderer->cr))
     {
@@ -240,8 +237,7 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
     g_free (cairo_glyphs);
 
 done:
-  if (!crenderer->do_path)
-    cairo_restore (crenderer->cr);
+  cairo_restore (crenderer->cr);
   
 #undef MAX_STACK
 }
@@ -287,8 +283,8 @@ pango_cairo_renderer_draw_rectangle (PangoRenderer     *renderer,
  *        \  /    \  /              \  /    \  /          |
  *         \/      \/                \/      \/           -
  *         B                         B       
- * |----|
- *   unit_width = (HEIGHT_SQUARES - 1) * square
+ *         |---|
+ *       unit_width = (HEIGHT_SQUARES - 1) * square
  *
  * The x, y, width, height passed in give the desired bounding box;
  * x/width are adjusted to make the underline a integer number of units
@@ -296,7 +292,6 @@ pango_cairo_renderer_draw_rectangle (PangoRenderer     *renderer,
  */
 #define HEIGHT_SQUARES 2.5
 
-/* This code is cut-and-pasted between here and gtk+/gdk/gdkpango.c */
 static void
 draw_error_underline (cairo_t *cr,
 		      double   x,
@@ -310,7 +305,7 @@ draw_error_underline (cairo_t *cr,
   double y_top, y_bottom;
   int i;
 
-  x += (width - width_units * unit_width);
+  x += (width - width_units * unit_width) / 2;
   width = width_units * unit_width;
 
   y_top = y;
@@ -399,6 +394,129 @@ pango_cairo_renderer_class_init (PangoCairoRendererClass *klass)
   renderer_class->draw_error_underline = pango_cairo_renderer_draw_error_underline;
 }
 
+
+/* convenience wrappers using the default renderer */
+
+
+static void
+_pango_cairo_do_glyph_string (cairo_t          *cr,
+			      PangoFont        *font,
+			      PangoGlyphString *glyphs,
+			      gboolean          do_path)
+{
+  PangoFontMap *fontmap;
+  PangoCairoRenderer *crenderer;
+  PangoRenderer *renderer;
+  gboolean unref_renderer = FALSE;
+
+  fontmap = pango_font_get_font_map (font);
+  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
+  if (G_UNLIKELY (!renderer))
+    {
+      renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
+      unref_renderer = TRUE;
+    }
+
+  crenderer = PANGO_CAIRO_RENDERER (renderer);
+
+  crenderer->cr = cr;
+  crenderer->do_path = do_path;
+  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
+  
+  pango_renderer_draw_glyphs (renderer, font, glyphs, 0, 0);
+  
+  if (G_UNLIKELY (unref_renderer))
+    g_object_unref (renderer);
+  else
+    {
+      crenderer->cr = NULL;
+      crenderer->do_path = FALSE;
+      crenderer->x_offset = 0.;
+      crenderer->y_offset = 0.;
+    }
+}
+
+static void
+_pango_cairo_do_layout_line (cairo_t          *cr,
+			     PangoLayoutLine  *line,
+			     gboolean          do_path)
+{
+  PangoContext *context;
+  PangoFontMap *fontmap;
+  PangoRenderer *renderer;
+  PangoCairoRenderer *crenderer;
+
+  context = pango_layout_get_context (line->layout);
+  fontmap = pango_context_get_font_map (context);
+  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
+  crenderer = PANGO_CAIRO_RENDERER (renderer);
+
+  crenderer->cr = cr;
+  crenderer->do_path = do_path;
+  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
+
+  pango_renderer_draw_layout_line (renderer, line, 0, 0);
+  
+  crenderer->cr = NULL;
+  crenderer->do_path = FALSE;
+  crenderer->x_offset = 0.;
+  crenderer->y_offset = 0.;
+}
+
+static void
+_pango_cairo_do_layout (cairo_t     *cr,
+			PangoLayout *layout,
+			gboolean     do_path)
+{
+  PangoContext *context;
+  PangoFontMap *fontmap;
+  PangoRenderer *renderer;
+  PangoCairoRenderer *crenderer;
+
+  context = pango_layout_get_context (layout);
+  fontmap = pango_context_get_font_map (context);
+  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
+  crenderer = PANGO_CAIRO_RENDERER (renderer);
+  
+  crenderer->cr = cr;
+  crenderer->do_path = do_path;
+  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
+
+  pango_renderer_draw_layout (renderer, layout, 0, 0);
+  
+  crenderer->cr = NULL;
+  crenderer->do_path = FALSE;
+  crenderer->x_offset = 0.;
+  crenderer->y_offset = 0.;
+}
+
+static void 
+_pango_cairo_do_error_underline (cairo_t *cr,                                  
+				 double   x,
+				 double   y,
+				 double   width,
+				 double   height,
+				 gboolean do_path)
+{  
+  /* We don't use a renderer here, for a simple reason:
+   * the only renderer we can get is the default renderer, that
+   * is all implemented here, so we shortcircuit and make our
+   * life way easier.
+   */
+
+  if (!do_path)
+    cairo_new_path (cr);
+
+  draw_error_underline (cr, x, y, width, height);
+
+  if (!do_path)
+    cairo_fill (cr);
+} 
+
+
+/* public wrapper of above to show or append path */
+
+
 /**
  * pango_cairo_show_glyph_string:
  * @cr: a Cairo context
@@ -416,51 +534,10 @@ pango_cairo_show_glyph_string (cairo_t          *cr,
 			       PangoFont        *font,
 			       PangoGlyphString *glyphs)
 {
-  PangoFontMap *fontmap;
-  PangoCairoRenderer *crenderer;
-  PangoRenderer *renderer;
-  gboolean unref_renderer = FALSE;
-
   g_return_if_fail (cr != NULL);
   g_return_if_fail (glyphs != NULL);
 
-  fontmap = pango_font_get_font_map (font);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  if (G_UNLIKELY (!renderer))
-    {
-      renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
-      unref_renderer = TRUE;
-    }
-
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-
-  cairo_save (cr);
-
-  crenderer->cr = cr;
-  crenderer->do_path = FALSE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-  
-  pango_renderer_activate (renderer);
-
-  pango_renderer_set_color (renderer, PANGO_RENDER_PART_FOREGROUND, NULL);
-  pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL);
-  pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL);
-  pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL);
-  
-  pango_renderer_draw_glyphs (renderer, font, glyphs, 0, 0);
-
-  pango_renderer_deactivate (renderer);
-  
-  if (G_UNLIKELY (unref_renderer))
-    g_object_unref (renderer);
-  else
-    {
-      crenderer->cr = NULL;
-      crenderer->x_offset = 0.;
-      crenderer->y_offset = 0.;
-    }
-  
-  cairo_restore (cr);
+  _pango_cairo_do_glyph_string (cr, font, glyphs, FALSE);
 }
 
 /**
@@ -478,32 +555,10 @@ void
 pango_cairo_show_layout_line (cairo_t          *cr,
 			      PangoLayoutLine  *line)
 {
-  PangoContext *context;
-  PangoFontMap *fontmap;
-  PangoRenderer *renderer;
-  PangoCairoRenderer *crenderer;
-
   g_return_if_fail (cr != NULL);
   g_return_if_fail (line != NULL);
 
-  context = pango_layout_get_context (line->layout);
-  fontmap = pango_context_get_font_map (context);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-
-  cairo_save (cr);
-
-  crenderer->cr = cr;
-  crenderer->do_path = FALSE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-
-  pango_renderer_draw_layout_line (renderer, line, 0, 0);
-  
-  crenderer->cr = NULL;
-  crenderer->x_offset = 0.;
-  crenderer->y_offset = 0.;
-  
-  cairo_restore (cr);
+  _pango_cairo_do_layout_line (cr, line, FALSE);
 }
 
 /**
@@ -521,34 +576,41 @@ void
 pango_cairo_show_layout (cairo_t     *cr,
 			 PangoLayout *layout)
 {
-  PangoContext *context;
-  PangoFontMap *fontmap;
-  PangoRenderer *renderer;
-  PangoCairoRenderer *crenderer;
-
   g_return_if_fail (cr != NULL);
   g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  context = pango_layout_get_context (layout);
-  fontmap = pango_context_get_font_map (context);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-  
-  cairo_save (cr);
-
-  crenderer->cr = cr;
-  crenderer->do_path = FALSE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-
-  pango_renderer_draw_layout (renderer, layout, 0, 0);
-  
-  crenderer->cr = NULL;
-  crenderer->x_offset = 0.;
-  crenderer->y_offset = 0.;
-  
-  cairo_restore (cr);
+  _pango_cairo_do_layout (cr, layout, FALSE);
 }
 
+/**
+ * pango_cairo_show_error_underline:
+ * @cr: a Cairo context
+ * @x: The X coordinate of one corner of the rectangle
+ * @y: The Y coordinate of one corner of the rectangle
+ * @width: Non-negative width of the rectangle
+ * @height: Non-negative height of the rectangle
+ * 
+ * Draw a squiggly line in the specified cairo context that approximately
+ * covers the given rectangle in the style of an underline used to indicate a
+ * spelling error.  (The width of the underline is rounded to an integer
+ * number of up/down segments and the resulting rectangle is centered in the
+ * original rectangle)
+ *
+ * Since: 1.14
+ **/
+void 
+pango_cairo_show_error_underline (cairo_t *cr,                                  
+			          double  x,
+			          double  y,
+			          double  width,
+			          double  height)
+{  
+  g_return_if_fail (cr != NULL);
+  g_return_if_fail ((width >= 0) && (height >= 0));  
+  
+  _pango_cairo_do_error_underline (cr, x, y, width, height, FALSE);                                 
+} 
+
 /**
  * pango_cairo_glyph_string_path
  * @cr: a Cairo context
@@ -566,41 +628,10 @@ pango_cairo_glyph_string_path (cairo_t          *cr,
 			       PangoFont        *font,
 			       PangoGlyphString *glyphs)
 {
-  PangoFontMap *fontmap;
-  PangoCairoRenderer *crenderer;
-  PangoRenderer *renderer;
-  gboolean unref_renderer = FALSE;
-
   g_return_if_fail (cr != NULL);
   g_return_if_fail (glyphs != NULL);
 
-  fontmap = pango_font_get_font_map (font);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  if (G_UNLIKELY (!renderer))
-    {
-      renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
-      unref_renderer = TRUE;
-    }
-
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-
-  crenderer->cr = cr;
-  crenderer->do_path = TRUE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-  
-  pango_renderer_draw_glyphs (renderer, font, glyphs, 0, 0);
-  
-  if (G_UNLIKELY (unref_renderer))
-    g_object_unref (renderer);
-  else
-    {
-      crenderer->cr = NULL;
-      crenderer->do_path = FALSE;
-      crenderer->x_offset = 0.;
-      crenderer->y_offset = 0.;
-    }
-  
-  cairo_set_font_face (cr, NULL);
+  _pango_cairo_do_glyph_string (cr, font, glyphs, TRUE);
 }
 
 /**
@@ -618,31 +649,10 @@ void
 pango_cairo_layout_line_path (cairo_t          *cr,
 			      PangoLayoutLine  *line)
 {
-  PangoContext *context;
-  PangoFontMap *fontmap;
-  PangoRenderer *renderer;
-  PangoCairoRenderer *crenderer;
-  
   g_return_if_fail (cr != NULL);
   g_return_if_fail (line != NULL);
 
-  context = pango_layout_get_context (line->layout);
-  fontmap = pango_context_get_font_map (context);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-
-  crenderer->cr = cr;
-  crenderer->do_path = TRUE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-
-  pango_renderer_draw_layout_line (renderer, line, 0, 0);
-  
-  crenderer->cr = NULL;
-  crenderer->do_path = FALSE;
-  crenderer->x_offset = 0.;
-  crenderer->y_offset = 0.;
-  
-  cairo_set_font_face (cr, NULL);
+  _pango_cairo_do_layout_line (cr, line, TRUE);
 }
 
 /**
@@ -660,30 +670,37 @@ void
 pango_cairo_layout_path (cairo_t     *cr,
 			 PangoLayout *layout)
 {
-  PangoContext *context;
-  PangoFontMap *fontmap;
-  PangoRenderer *renderer;
-  PangoCairoRenderer *crenderer;
-
   g_return_if_fail (cr != NULL);
   g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  context = pango_layout_get_context (layout);
-  fontmap = pango_context_get_font_map (context);
-  renderer = _pango_cairo_font_map_get_renderer (PANGO_CAIRO_FONT_MAP (fontmap));
-  crenderer = PANGO_CAIRO_RENDERER (renderer);
-  
-  crenderer->cr = cr;
-  crenderer->do_path = TRUE;
-  cairo_get_current_point (cr, &crenderer->x_offset, &crenderer->y_offset);
-
-  pango_renderer_draw_layout (renderer, layout, 0, 0);
-  
-  crenderer->cr = NULL;
-  crenderer->do_path = FALSE;
-  crenderer->x_offset = 0.;
-  crenderer->y_offset = 0.;
-  
-  cairo_set_font_face (cr, NULL);
+  _pango_cairo_do_layout (cr, layout, TRUE);
 }
 
+/**
+ * pango_cairo_error_underline_path:
+ * @cr: a Cairo context
+ * @x: The X coordinate of one corner of the rectangle
+ * @y: The Y coordinate of one corner of the rectangle
+ * @width: Non-negative width of the rectangle
+ * @height: Non-negative height of the rectangle
+ * 
+ * Add a squiggly line to the current path in the specified cairo context that
+ * approximately covers the given rectangle in the style of an underline used
+ * to indicate a spelling error.  (The width of the underline is rounded to an
+ * integer number of up/down segments and the resulting rectangle is centered
+ * in the original rectangle)
+ *
+ * Since: 1.14
+ **/
+void 
+pango_cairo_error_underline_path (cairo_t *cr,                                  
+			          double   x,
+			          double   y,
+			          double   width,
+			          double   height)
+{
+  g_return_if_fail (cr != NULL);
+  g_return_if_fail ((width >= 0) && (height >= 0));
+  
+  _pango_cairo_do_error_underline (cr, x, y, width, height, TRUE);                                 
+}
